/*
 * Softcam plugin to VDR (C++)
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___CAM_H
#define ___CAM_H

#include <vdr/thread.h>
#include "data.h"
#include "misc.h"

class cDevice;
class cEcmHandler;
class cEcmData;
class cLogger;
class cHookManager;
class cLogHook;
class cSids;

// ----------------------------------------------------------------

class cEcmCache : public cLoader, cMutex, cSimpleList<cEcmData> {
private:
  cEcmData *Exists(cEcmInfo *e);
public:
  cEcmCache(void);
  void Load(void);
  void New(cEcmInfo *e);
  int GetCached(cSimpleList<cEcmInfo> *list, const cSids *sids, int Source, int Transponder);
  void Delete(cEcmInfo *e);
  virtual bool Save(FILE *f);
  virtual bool ParseLine(const char *line, bool fromCache);
  };

extern cEcmCache ecmcache;

// ----------------------------------------------------------------

#define MAX_CAM_PIDS 64
#define MAX_HLR_PIDS 16
#define MAX_CW_IDX   16

class cCam : public cMutex {
private:
  int cardNum;
  cDevice *device;
  int camDev;
  bool softCSA;
  cSimpleList<cEcmHandler> handlerList;
  cHookManager *hookman;
  cLogger *logger;
  int source, transponder, numPids;
  unsigned short pids[MAX_CAM_PIDS];
  unsigned char types[MAX_CAM_PIDS];
  unsigned char indexMap[MAX_CW_IDX], lastCW[MAX_CW_IDX][2*8];
  cTimeMs lastDump;
  //
  int PID2SID(int pid, cSids *sids);
  int FindPid(int pid);
  int FindType(int type);
  cEcmHandler *GetHandler(cSids *sids, bool needZero);
  void RemHandler(cEcmHandler *handler);
  int GetFreeIndex(void);
public:
  cCam(const cDevice *dev, int CardNum, int CamDev);
  virtual ~cCam();
  // EcmHandler API
  void WriteCW(int index, unsigned char *cw, bool force);
  void SetCWIndex(int pid, int index);
  void LogEcmStatus(const cEcmInfo *ecm, bool on);
  void AddHook(cLogHook *hook);
  bool TriggerHook(int id);
  void Dump(void);
  // Plugin API
  bool Active(void);
  void HouseKeeping(void);
  int ProvidesCa(const unsigned short *ids);
  bool ConcurrentAllowed(void);
  bool LiveViewActive(void);
  void Tune(int Source, int Transponder);
  void AddPid(int pid, int type);
  void DelPid(int pid);
  char *CurrentKeyStr(int num);
  //
  bool IsSoftCSA(void) { return softCSA; }
  int Source(void) { return source; }
  int Transponder(void) { return transponder; }
  };

void LogStatsDown(void);

#endif // ___CAM_H
