
#include <stdlib.h>
#include <stdio.h>

#define TESTER
#include "crypto.h"
#include "data.h"
#include "system-common.h"
#include "systems/nagra/cpu.c"
#include "systems/nagra/nagra2.c"

#include "compat.h"

int main(int argc, char *argv[])
{
  if(argc<3) {
    printf("usage: %s <plugin-dir> <id> <rawemm-file>\n",argv[0]);
    return 1;
    }

  InitAll(argv[1]);
  unsigned char emmdata[96];
  int cmdLen=ReadRaw(argv[3],emmdata,sizeof(emmdata));
  if(cmdLen!=96) {
    printf("bad rawemm file format\n");
    return 1;
    }

  int id=strtol(argv[2],0,0);
  cN2Prov *emmP=0;

  for(int i=8+2+4+4; i<cmdLen-22; ) {
printf("%02x: nano %02x\n",i,emmdata[i]);
    switch(emmdata[i]) {
      case 0x42: // plain Key update
        if(emmdata[i+2]==0x10 && (emmdata[i+3]&0xBF)==0x06 &&
           (emmdata[i+4]&0xF8)==0x08 && emmdata[i+5]==0x00 && emmdata[i+6]==0x10) {
          printf("key%02x: ",(emmdata[i+3]&0x40)>>6);
          SDump(&emmdata[i+7],16);
          }
        i+=23;
        break;
      case 0xE0: // DN key update
        if(emmdata[i+1]==0x25) {
          printf("key%02x: ",(emmdata[i+16]&0x40)>>6);
          SDump(&emmdata[i+23],16);
          }
        i+=39;
        break;
      case 0x83: // change data prov. id
        id=(emmdata[i+1]<<8)|emmdata[i+2];
        printf("keyid: %04x\n",id);
        i+=3;
        break;
      case 0xA4: // conditional (always no match assumed for now)
        i+=emmdata[i+1]+2+4;
        break;
      case 0xA6:
        i+=emmdata[i+1]+1;
        break;
      case 0x13:
      case 0x14:
      case 0x15:
        i+=4;
        break;
      case 0xB0: // Update with ROM101 CPU code
        i+=6;
        break;
      case 0xB1: // Update with ROM102 CPU code
        {
        if(!emmP || !emmP->CanHandle(id)) {
          delete emmP;
          emmP=cN2Providers::GetProv(id,2);
          if(!emmP) {
            dl(printf("system-nagra2: B1 for provider %04x not supported\n",id))
            i=cmdLen;
            break;
            }
          }
        int r;
        if((r=emmP->ProcessB1(emmdata,cmdLen,i+1))>0)
          i+=r;
        else {
          dl(printf("system-nagra2: B1 executing failed for %04x\n",id))
          i=cmdLen;
          }
        break;
        }
      case 0x00: // end of processing
        i=cmdLen;
        break;
      default:
        i++;
        continue;
      }
    }
}
